@@ -24,6 +24,7 @@ from __future__ import unicode_literals
 
 import email
 import io
+import json
 import os
 import re
 import subprocess
@@ -776,6 +777,49 @@ def have_sphinx():
     return _have_sphinx
 
 
+def _get_raw_tag_info(git_dir):
+    describe = _run_git_command(['describe', '--always'], git_dir)
+    if "-" in describe:
+        return describe.rsplit("-", 2)[-2]
+    if "." in describe:
+        return 0
+    return None
+
+
+def get_is_release(git_dir):
+    return _get_raw_tag_info(git_dir) == 0
+
+
+def _run_git_functions():
+    git_dir = _get_git_directory()
+    if git_dir and _git_is_installed():
+        return git_dir
+    return None
+
+
+def get_git_short_sha(git_dir=None):
+    """Return the short sha for this repo, if it exists."""
+    if not git_dir:
+        git_dir = _run_git_functions()
+    if git_dir:
+        return _run_git_command(
+            ['log', '-n1', '--pretty=format:%h'], git_dir)
+    return None
+
+
+def write_pbr_json(cmd, basename, filename):
+    git_dir = _run_git_functions()
+    if not git_dir:
+        return
+    values = dict()
+    git_version = get_git_short_sha(git_dir)
+    is_release = get_is_release(git_dir)
+    if git_version is not None:
+        values['git_version'] = git_version
+        values['is_release'] = is_release
+        cmd.write_file('pbr', filename, json.dumps(values))
+
+
 def _get_revno(git_dir):
     """Return the number of commits since the most recent tag.
 
@@ -783,9 +827,9 @@ def _get_revno(git_dir):
     tags then we fall back to counting commits since the beginning
     of time.
     """
-    describe = _run_git_command(['describe', '--always'], git_dir)
-    if "-" in describe:
-        return describe.rsplit("-", 2)[-2]
+    raw_tag_info = _get_raw_tag_info(git_dir)
+    if raw_tag_info:
+        return raw_tag_info
 
     # no tags found
     revlist = _run_git_command(
@@ -799,21 +843,30 @@ def _get_version_from_git(pre_version):
     if the current revision has no tag.
     """
 
-    git_dir = _get_git_directory()
-    if git_dir and _git_is_installed():
+    git_dir = _run_git_functions()
+    if git_dir:
         if pre_version:
             try:
                 return _run_git_command(
                     ['describe', '--exact-match'], git_dir,
                     throw_on_error=True).replace('-', '.')
             except Exception:
-                sha = _run_git_command(
-                    ['log', '-n1', '--pretty=format:%h'], git_dir)
-                return "%s.dev%s+g%s" % (pre_version, _get_revno(git_dir), sha)
+                return "%s.dev%s" % (pre_version, _get_revno(git_dir))
         else:
-            return _run_git_command(
-                ['describe', '--always'],
-                git_dir).replace('-', '.').replace('.g', '+g')
+            # git describe always is going to return one of three things
+            # - a short-sha if there are no tags
+            # - a tag, if there's one on the current revision
+            # - a string of the form $last_tag-$revs_since_last_tag-g$short_sha
+            raw_version = _run_git_command(['describe', '--always'], git_dir)
+            # First, if there are no -'s or .'s, then it's just a short sha.
+            # Create a synthetic version for it.
+            if '-' not in raw_version and '.' not in raw_version:
+                return "0.0.0.%s" % _get_revno(git_dir)
+            # Now, we want to strip the short-sha prefix
+            stripped_version = raw_version.split('-g')[0]
+            # Finally, if we convert - to ., we'll get the version we want
+            return stripped_version.replace('-', '.')
+
     # If we don't know the version, return an empty string so at least
     # the downstream users of the value always have the same type of
     # object to work with.
